# frozen_string_literal: true

module ServerComponentHelper
  class SetStateDSL
    def initialize(jsrb, new_state, prev_state)
      @js = jsrb
      @new_state = new_state
      @prev_state = prev_state
    end

    # rubocop:disable Style/MethodMissingSuper
    def method_missing(name, *args)
      if (matches = name.to_s.match(/^(.+)=$/))
        @js.set! @new_state[matches[1]], args[0]
      elsif block_given?
        block_result = yield @prev_state[name]
        @js.set! @new_state[name], block_result
      else
        @new_state[name]
      end
    end
    # rubocop:enable Style/MethodMissingSuper
  end

  def set_state(jsrb) # rubocop:disable Naming/AccessorMethodName
    new_state = jsrb.var! { {} }
    prev_state = jsrb.expr.component.state
    yield SetStateDSL.new(jsrb, new_state, prev_state)
    jsrb.do! jsrb.expr.component.setState.call(new_state)
  end
end
