# frozen_string_literal: true

# Use this hook to configure ServerComponent
ServerComponent.setup do |config|
  # context_script is prepended to response scripts.
  # It is useful if you want to add functions
  # invoked from JSRB renderer.
  # You can get identifiers from JSRB by calling
  #  `js.identifier_name` .
  # The combination of 'add_custom_chain' option will
  # make your JSRB code more readable.
  config.context_script = <<~JAVASCRIPT
    // Returns new object or array
    // updating a property by immediate value or updator
    // function in the specified path.
    // without side effect in the original.
    //
    // example:
    //   immutable_update({ foo: 1 }, 'foo', 2) => { foo: 2 }
    //
    var immutable_update = (function () {
      function _is_array(target) {
        return Object.prototype.toString.call(target) === '[object Array]';
      }

      function _is_object(target) {
        return typeof target === 'object' && target !== null;
      }

      function _clone_array_or_object(target) {
        if (_is_array(target)) {
          return target.slice(0);
        } else if (_is_object(target)) {
          return Object(target);
        } else {
          throw 'Argument Error: target is neither object nor array';
        }
      }

      function _clone_update_internal(obj, field, value) {
        if (value instanceof Function) {
          obj[field] = value(obj[field]);
        } else {
          obj[field] = value;
        }
      }

      return function (target, field, updator) {
        var clone = _clone_array_or_object(target);
        _clone_update_internal(clone, field, updator);
        return clone;
      }
    })();
  JAVASCRIPT

  # Adds functionality to JSRB DSL.
  # If you add 'func!' to 'some_func',
  # then `obj.func!(arg)` is equivalent to `js.some_func.(obj, arg)`
  # where `obj` is a chain instance in JSRB.
  config.add_custom_chain(:update!, :immutable_update)

  # Specify parent class of ServerComponent's controllers.
  # config.parent_controller = 'ApplicationController'
end
