# frozen_string_literal: true

module ServerComponent
  class ComponentRouter
    attr_reader :namespace, :components

    def initialize(namespace = nil)
      @namespace = namespace
      @components = []
    end

    def to_json
      h = {}
      h[:components] = components.each_with_object({}) do |component, json|
        component_controller_class = find_component_controller_class(component)
        json[component] = component_controller_class.to_json
      end
      h[:context_script] = context_script
      h
    end

    def add_component(name)
      @components << name.to_sym
    end

    def context_script
      ServerComponent.context_script
    end

    protected

    def find_component_controller_class(component)
      name = (namespace ? "#{namespace}/#{component}" : component).to_s
      "#{name.camelize}ComponentController".constantize
    end
  end
end
